class CommonLicensesController < ApplicationController
  layout 'test' if Pettanr::TestLayout
  before_filter :authenticate_user!, :only => [:index, :show]
  before_filter :authenticate_admin!, :only => [:list, :browse, :import, :destroy]

  private
  
  def set_data(file)
    if file.respond_to?(:read)
      file.read
    else
      file
    end
  end
  
  public
  
  # GET /common_licenses
  # GET /common_licenses.json
  def index
    @page = CommonLicense.page params[:page]
    @page_size = CommonLicense.page_size params[:page_size]
    @common_licenses = CommonLicense.list({}, @page, @page_size)

    respond_to do |format|
      format.html # index.html.erb
      format.json { render json: @common_licenses.to_json(CommonLicense.list_json_opt) }
    end
  end

  # GET /common_licenses/1
  # GET /common_licenses/1.json
  def show
    @common_license = CommonLicense.show(params[:id])

    respond_to do |format|
      format.html # show.html.erb
      format.json { render json: @common_license.to_json(CommonLicense.show_json_include_opt) }
    end
  end

  def list
    @common_licenses = CommonLicense.all

    respond_to do |format|
      format.html { render layout: 'system' }
      format.json { render json: @common_licenses }
    end
  end

  def browse
    @common_license = CommonLicense.find(params[:id])

    respond_to do |format|
      format.html { render layout: 'system' }
      format.json { render json: @common_license }
    end
  end

  # POST /common_licenses
  # POST /common_licenses.json
  def import
    @data = set_data params[:file]
    respond_to do |format|
      @errors = CommonLicense.import @data
      if @errors and @errors.empty?
        format.html { redirect_to :action => :list }
        format.json { render text: 'ok', status: :created }
      else
        format.html { render action: "result" }
        format.json { render json: @errors, status: :unprocessable_entity }
      end
    end
  end

  # DELETE /common_licenses/1
  # DELETE /common_licenses/1.json
  def destroy
    @common_license = CommonLicense.find(params[:id])
    @common_license.destroy

    respond_to do |format|
      format.html { redirect_to common_licenses_url }
      format.json { head :ok }
    end
  end
end
