class Comic < ActiveRecord::Base
  has_many :panels, :dependent => :destroy
  belongs_to :author
  
  validates :title, :presence => true, :length => {:maximum => 100}
  validates :width, :presence => true, :numericality => true, :natural_number => true
  validates :height, :presence => true, :numericality => true, :natural_number => true
  validates :visible, :presence => true, :numericality => true, :inclusion => {:in => 0..3}
  validates :editable, :presence => true, :numericality => true, :inclusion => {:in => 0..3}
  
  before_save do |r|
    r.supply_default
  end
  
  def supply_default
    self.visible = 0 if self.visible.blank?
    self.editable= 0 if self.editable.blank?
  end
  
  def own? author
    return false unless author
    self.author_id == author.id
  end
  
  def visible? author
    self.visible > 0 or self.own?(author)
  end
  
  def disp_editable
    editable == 1 ? 'O' : 'X'
  end
  
  def disp_visible
    visible == 1 ? 'O' : 'X'
  end
  
  def self.default_page_size
    25
  end
  
  def self.max_page_size
    100
  end
  
  def self.default_panel_size
    30
  end
  
  def self.max_panel_size
    200
  end
  
  def self.page prm = nil
    page = prm.to_i
    page = 1 if page < 1
    page
  end
  
  def self.page_size prm = self.default_page_size
    page_size = prm.to_i
    page_size = self.max_page_size if page_size > self.max_page_size
    page_size = self.default_page_size if page_size < 1
    page_size
  end
  
  def self.offset cnt, prm = nil
    offset = prm.to_i
    offset = cnt - 1 if offset >= cnt
    offset = cnt - offset.abs if offset < 0
    offset = 0 if offset < 0
    offset
  end
  
  def self.panel_count cnt, prm = self.default_panel_size
    count = prm.to_i
    count = self.max_panel_size if count > self.max_panel_size
    count = self.default_panel_size if count < 1
    count
  end
  
  def self.list opt = {}, page = 1, page_size = self.default_page_size
    opt.merge!(self.list_opt) unless opt[:include]
    opt.merge!({:conditions => ['visible > 0'], :order => 'updated_at desc', :limit => page_size, :offset => (page -1) * page_size})
    Comic.find(:all, opt)
  end
  
  def self.list_opt
    {:include => :author}
  end
  
  def self.list_json_opt
    {:include => :author}
  end
  
  def self.show cid, au, opt = {}
    c = Comic.find(cid, :include => self.show_include_opt(opt))
    raise ActiveRecord::Forbidden unless c.visible?(au)
    c
  end
  
  def self.show_include_opt opt = {}
    res = [:author]
    res.push(opt[:include]) if opt[:include]
    res
  end
  
  def self.show_json_include_opt
    {:include => :author}
  end
  
  def self.play cid, opt = {}
    Comic.find(cid, include: [
      :author, 
      :panels => [
        :panel_pictures => :resource_picture, 
        :balloons => :speaches
      ]
    ], order: 'panels.t, speaches.t')
  end
  
  def to_json_play
    self.to_json( :include => {:author => {}, :panels => {:methods => :panel_element}})
  end
  
  def self.visible_count
    Comic.count 'visible > 0'
  end
  
end
