# -*- encoding: utf-8 -*-
#素材
require 'spec_helper'

describe ResourcePicturesController do
  before do
    Factory :admin
    @license = Factory :license
    @user = Factory( :user_yas)
    @author = @user.author
    @artist = Factory :artist_yas, :author_id => @author.id
    @op = Factory :original_picture, :artist_id => @artist.id , :license_id => @license.id
  end

  describe '一覧表示に於いて' do
    before do
      @rp = Factory :resource_picture, :artist_id => @artist.id , :license_id => @license.id, :original_picture_id => @op.id
      sign_in @user
      ResourcePicture.stub(:list).and_return([@rp, @rp, @rp])
    end
    context 'パラメータpageについて' do
      it '@pageに値が入る' do
        get :index, :page => 5
        assigns(:page).should eq 5
      end
      it '省略されると@pageに1値が入る' do
        get :index
        assigns(:page).should eq 1
      end
      it '与えられたpage_sizeがセットされている' do
        get :index, :page_size => 15
        assigns(:page_size).should eq 15
      end
      it '省略されると@page_sizeにデフォルト値が入る' do
        get :index
        assigns(:page_size).should eq ResourcePicture.default_page_size
      end
      it '最大を超えると@page_sizeにデフォルト最大値が入る' do
        get :index, :page_size => 1500
        assigns(:page_size).should eq ResourcePicture.max_page_size
      end
      it '不正な値が入ると@page_sizeにデフォルト最大値が入る' do
        get :index, :page_size => 0
        assigns(:page_size).should eq ResourcePicture.default_page_size
      end
    end
    context 'つつがなく終わるとき' do
      it 'ステータスコード200 OKを返す' do
        get :index
        response.should be_success 
      end
      it '素材モデルに一覧を問い合わせている' do
        ResourcePicture.should_receive(:list).exactly(1)
        get :index
      end
      it '@resource_picturesにリストを取得している' do
        get :index
        assigns(:resource_pictures).should have_at_least(3).items
      end
      context 'html形式' do
        it 'indexテンプレートを描画する' do
          get :index
          response.should render_template("index")
        end
      end
      context 'json形式' do
        it 'jsonデータを返す' do
          get :index, :format => :json
          lambda{JSON.parse(response.body)}.should_not raise_error(JSON::ParserError)
        end
        it 'データがリスト構造になっている' do
          get :index, :format => :json
          json = JSON.parse response.body
          json.should have_at_least(3).items
        end
        it 'リストの先頭くらいは素材っぽいものであって欲しい' do
          get :index, :format => :json
          json = JSON.parse response.body
          json.first.has_key?("ext").should be_true
        end
      end
    end
    context '作家権限がないとき' do
      before do
        sign_out @user
      end
      context 'html形式' do
        it 'ステータスコード302 Foundを返す' do
          get :index
          response.status.should eq 302
        end
        it 'サインインページへ遷移する' do
          get :index
          response.should redirect_to '/users/sign_in'
        end
      end
      context 'json形式' do
        it 'ステータスコード401 Unauthorizedを返す' do
          get :index, :format => :json
          response.status.should eq 401
        end
        it '応答メッセージにUnauthorizedを返す' do
          get :index, :format => :json
          response.message.should match(/Unauthorized/)
        end
      end
    end
  end
  
  describe '単体表示に於いて' do
    before do
      @pic = Factory :resource_picture, :artist_id => @artist.id , :license_id => @license.id, :original_picture_id => @op.id
      sign_in @user
      ResourcePicture.stub(:show).and_return(@pic)
    end
    context 'つつがなく終わるとき' do
      it 'ステータスコード200 OKを返す' do
        get :show, :id => @pic.id
        response.should be_success
      end
      it '素材モデルに単体取得を問い合わせている' do
        ResourcePicture.should_receive(:show).exactly(1)
        get :show
      end
      it '@resource_pictureにアレを取得している' do
        get :show, :id => @pic.id
        assigns(:resource_picture).id.should eq(@pic.id)
      end
      context 'html形式' do
        it 'showテンプレートを描画する' do
          get :show, :id => @pic.id
          response.should render_template("show")
        end
      end
      context 'json形式' do
        it 'jsonデータを返す' do
          get :show, :id => @pic.id, :format => :json
          lambda{JSON.parse(response.body)}.should_not raise_error(JSON::ParserError)
        end
        it 'データがアレになっている' do
          get :show, :id => @pic.id, :format => :json
          json = JSON.parse response.body
          json["ext"].should match(/png/)
        end
      end
      #画像送信では、send_dataにスタブをおいてテストしたいが、ここに噛ませると
      #renderが働かず、エラーとなってしまう。そこで、素材のファイル取得部分に
      #スタブをおいてsend_dataがデータを返す体裁でテストする。
      context 'png形式' do
        it '画像モデルに画像データを問い合わせる' do
          ResourcePicture.any_instance.should_receive(:restore).exactly(1)
          get :show, :id => @pic.id, :format => :png
        end
        it '画像を送信する' do
          ResourcePicture.any_instance.stub(:restore).and_return('aaa')
          get :show, :id => @pic.id, :format => :png
          response.body.should eq 'aaa'
        end
      end
      context 'gif形式' do
        it '画像モデルに画像データを問い合わせる' do
          ResourcePicture.any_instance.should_receive(:restore).exactly(1)
          get :show, :id => @pic.id, :format => :gif
        end
        it '画像を送信する' do
          ResourcePicture.any_instance.stub(:restore).and_return('bbb')
          get :show, :id => @pic.id, :format => :gif
          response.body.should eq 'bbb'
        end
      end
      context 'jpeg形式' do
        it '画像モデルに画像データを問い合わせる' do
          ResourcePicture.any_instance.should_receive(:restore).exactly(1)
          get :show, :id => @pic.id, :format => :jpeg
        end
        it '画像を送信する' do
          ResourcePicture.any_instance.stub(:restore).and_return('ccc')
          get :show, :id => @pic.id, :format => :jpeg
          response.body.should eq 'ccc'
        end
      end
    end
    context '作家権限がないとき' do
      before do
        sign_out @user
      end
      context 'html形式' do
        it 'ステータスコード302 Foundを返す' do
          get :show, :id => @pic.id
          response.status.should eq 302
        end
        it 'サインインページへ遷移する' do
          get :show, :id => @pic.id
          response.body.should redirect_to '/users/sign_in'
        end
      end
      context 'json形式' do
        it 'ステータスコード401 Unauthorizedを返す' do
          get :show, :id => @pic.id, :format => :json
          response.status.should eq 401
        end
        it '応答メッセージにUnauthorizedを返す' do
          get :show, :id => @pic.id, :format => :json
          response.message.should match(/Unauthorized/)
        end
      end
    end
=begin
    context '対象素材がないとき' do
      before do
        ResourcePicture.unstub(:show)
      end
      context 'html形式' do
        it '例外404 not_foundを返す' do
          lambda{
            get :show, :id => 0
          }.should raise_error(ActiveRecord::RecordNotFound)
        end
      end
      context 'json形式' do
        it '例外404 not_foundを返す' do
          lambda{ 
            get :show, :id => 0, :format => :json
          }.should raise_error(ActiveRecord::RecordNotFound)
        end
      end
    end
    context '他人の素材を見ようとしたとき' do
      before do
        ResourcePicture.stub(:show).and_return(@pic)
        ResourcePicture.any_instance.stub(:own?).with(any_args()).and_return(false)
      end
      context 'html形式' do
        it '例外403 forbiddenを返す' do
          lambda{
            get :show, :id => @pic.id
          }.should raise_error(ActiveRecord::Forbidden)
        end
      end
      context 'json形式' do
        it '例外403 forbiddenを返す' do
          lambda{
            get :show, :id => @pic.id, :format => :json
          }.should raise_error(ActiveRecord::Forbidden)
        end
      end
    end
=end
  end

  describe '素材数取得に於いて' do
    before do
      ResourcePicture.should_receive(:visible_count).and_return(3)
#      sign_in @user
    end
    context 'つつがなく終わるとき' do
      it 'ステータスコード200 OKを返す' do
        get :count, :format => :json
        response.should be_success 
      end
      context 'json形式' do
        it 'jsonデータを返す' do
          get :count, :format => :json
          lambda{JSON.parse(response.body)}.should_not raise_error(JSON::ParserError)
        end
        it 'データがHash構造になっていてコマ絵数が3である' do
          get :count, :format => :json
          json = JSON.parse response.body
          json["count"].should == 3
        end
      end
    end
  end

end
